@@ -18,7 +18,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 #include "dbus-spawn.h"
@@ -74,8 +74,8 @@ read_ints (int        fd,
   
   while (TRUE)
     {
-      size_t chunk;
-      ssize_t to_read;
+      ssize_t chunk;
+      size_t to_read;
 
       to_read = sizeof (int) * n_ints_in_buf - bytes;
 
@@ -129,9 +129,9 @@ read_pid (int        fd,
   
   while (TRUE)
     {
-      size_t chunk;    
-      ssize_t to_read;
-      
+      ssize_t chunk;
+      size_t to_read;
+
       to_read = sizeof (pid_t) - bytes;
 
       if (to_read == 0)
